use crate::client::ONLINE_USERS;
use crate::utils::json_utils::get_current_timestamp;
use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct OnlineUserMessage {
    #[serde(rename = "callState")]
    pub call_state: String,
    #[serde(rename = "channelID")]
    pub channel_id: String,
    #[serde(rename = "deviceID")]
    pub device_id: String,
    #[serde(rename = "fromID")]
    pub from_id: String,
    #[serde(rename = "hasCamera")]
    pub has_camera: String,
    #[serde(rename = "hasMike")]
    pub has_mike: String,
    #[serde(rename = "isHost")]
    pub is_host: String,
    #[serde(rename = "userCallGroup")]
    pub user_call_group: String,
    #[serde(rename = "fromName")]
    pub from_name: String,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct OnlineUsersMessage {
    #[serde(rename = "msgType")]
    pub msg_type: String,
    #[serde(rename = "fromId")]
    pub from_id: String,
    #[serde(rename = "fromName")]
    pub from_name: String,
    #[serde(rename = "toId")]
    pub to_id: String,
    #[serde(rename = "msgData")]
    pub msg_data: Vec<OnlineUserMessage>,
    #[serde(rename = "time")]
    pub time: u128,
}

pub async fn send_online_users_resp() -> Result<Vec<(String, String)>, serde_json::Error> {
    let mut msg_data = Vec::new();
    let mut user_ids = Vec::new();

    {
        let online_users = ONLINE_USERS.read().await;
        println!("当前所有用户数据信息 ONLINE_USERS: {:?}", online_users);
        for (user_id, user_info_str) in online_users.iter() {
            let parts: Vec<&str> = user_info_str.split(',').collect();
            if parts.len() == 9 {
                let user_msg = OnlineUserMessage {
                    call_state: parts[0].to_string(),
                    channel_id: parts[1].to_string(),
                    device_id: parts[2].to_string(),
                    from_id: parts[3].to_string(),
                    has_camera: parts[4].to_string(),
                    has_mike: parts[5].to_string(),
                    is_host: parts[6].to_string(),
                    user_call_group: parts[7].to_string(),
                    from_name: parts[8].to_string(),
                };
                msg_data.push(user_msg);
                user_ids.push(user_id.clone());
            }
        }
    }

    let mut messages = Vec::new();
    for user_id in user_ids {
        let to_id = user_id.clone();
        let message = OnlineUsersMessage {
            msg_type: "CmdUpdateOnlineUsers".to_string(),
            from_id: "0".to_string(),
            from_name: "Server".to_string(),
            to_id,
            msg_data: msg_data.clone(),
            time: get_current_timestamp(),
        };

        let json = serde_json::to_string(&message)?;
        messages.push((user_id, json));
    }

    Ok(messages)
}
